///////////////////////////////////////////////////////////////////////////////
//                                                         
// Datetime.cc
// -----------
// implementation of a datetime data type
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2013 Bjoern Lemke                        
//
// IMPLEMENTATION MODULE
//
// Class: Datetime
// 
// Description: datetime utility class
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>

#include "Exception.h"
#include "Datetime.h"

#define BUF 128
#define DTFORMAT "%d.%m.%Y %H:%M:%S"
// old def : "%a %b %d %H:%M:%S %Y"
// on windows, the zoneinfo looks uggly
// #define DTFORMAT "%a %b %d %H:%M:%S %Y"

Datetime::Datetime()
{    
    time ( &_t );
}

Datetime::Datetime(const Chain& v)
{   

    struct tm *pTM;

    pTM=(struct tm*)malloc(sizeof(struct tm));
    if (pTM == NULL)
    {
	throw Exception (EXLOC, "malloc failed");
    }

    if ( strptime((char*)v, DTFORMAT , pTM) == 0 )
	throw Exception (EXLOC, Chain("Invalid datetime string <") + v + Chain("> for default format <") + Chain(DTFORMAT) + Chain(">"));

    pTM->tm_isdst=-1;

    tzset();
    
    _t = mktime(pTM);
    free(pTM);

}

Datetime::Datetime(const Chain& v, const Chain& format)
{    
    struct tm *pTM;

    pTM=(struct tm*)malloc(sizeof(struct tm));
    if (pTM == NULL)
    {
	throw Exception (EXLOC, "malloc failed");
    }
    if ( strptime((char*)v, (char*)format , pTM) == 0 )
	throw Exception (EXLOC, Chain("Invalid datetime string <") + v + Chain("> for format <") + format + Chain(">"));

    pTM->tm_isdst=-1;
    tzset();
    _t = mktime(pTM);

    free(pTM);
}

Datetime::Datetime(int i)
{
    _t = (time_t)i;
}

Datetime::~Datetime()
{
}

Chain Datetime::asChain() const
{
    char buf[BUF];

    tzset();
    struct tm *pTM = localtime( &_t );
    // struct tm *pTM = gmtime( &_t );

    strftime(buf,BUF,DTFORMAT,pTM);

    Chain s( buf );
    return s;
}

Chain Datetime::asChain(const Chain& format) const
{
    char buf[BUF];

    struct tm *pTM = localtime( &_t );

    strftime(buf,BUF, (char*)format, pTM);

    Chain s( buf );
    return s;
}

int Datetime::asInt() const
{
    return (int)_t;
}

void Datetime::add(int seconds)
{
    _t += seconds;
}

void Datetime::sub(int seconds)
{
    _t = _t - seconds;
}

Datetime& Datetime::operator=(const Datetime& d)
{     
    _t = d._t;
    return (*this);
}
    
bool Datetime::operator==(const Datetime& d) const
{
    if ( _t == d._t )
	return true;
    return false;
}
    
bool Datetime::operator!=(const Datetime& d) const
{
    if ( _t != d._t )
	return true;
    return false;
}

bool Datetime::operator < (const Datetime& d) const
{
    if ( _t < d._t )
	return true;
    return false;
}

bool Datetime::operator > (const Datetime& d) const
{
    if ( _t > d._t )
	return true;
    return false;
}

bool Datetime::operator <= (const Datetime& d) const
{
    if ( _t <= d._t )
	return true;
    return false;
}

bool Datetime::operator >= (const Datetime& d) const
{
    if ( _t >= d._t )
	return true;
    return false;
}
    
ostream& operator << (ostream& s, const  Datetime& d)
{
    s << d.asChain();
    return s;
}













