///////////////////////////////////////////////////////////////////////////////
//                                                         
// Sleeper.cc
// ----------
// Sleeper class implementation
//                                               
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2013 Bjoern Lemke                        
//
// IMPLEMENTATION MODULE
//
// Class: Sleeper
// 
// Description: all operations on network resources
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#ifndef _REENTRANT
#define _REENTRANT    /* basic 3-lines for threads */
#endif


#include "Sleeper.h"

#ifdef HAVE_MINGW32
#include <windows.h>
#endif

// SYSTEM INCLUDES
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "Exception.h"
#include "Chain.h"


Sleeper::Sleeper()
{
}

Sleeper::~Sleeper()
{
}

void Sleeper::secSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n * 1000);
#else
    ::sleep(n);
#endif
}

void Sleeper::milliSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n);
#else
    struct timespec delay;	
    delay.tv_sec = 0; 
    delay.tv_nsec = n * 1000000;
    if ( nanosleep(&delay, NULL) == -1 )
    {
	Chain msg = Chain("nanosleep system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
#endif
}

void Sleeper::microSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n / 1000);
#else
    struct timespec delay;	
    delay.tv_sec = 0; 
    delay.tv_nsec = n * 1000;
    if ( nanosleep(&delay, NULL) == -1 )
    {
	Chain msg = Chain("nanosleep system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
#endif
}

void Sleeper::nanoSleep(int n) const
{
#ifdef HAVE_MINGW32
    Sleep(n / 1000000);
#else

    struct timespec delay;	
    delay.tv_sec = 0; 
    delay.tv_nsec = n;
    if ( nanosleep(&delay, NULL) == -1 )
    {
	Chain msg = Chain("nanosleep system error : ") + Chain(strerror(errno));
	throw Exception(EXLOC, msg);
    }
#endif
}




