///////////////////////////////////////////////////////////////////////////////
//
// Thread.cc 
// ---------
// Thread class implementation
//                                                
// Design and Implementation by Bjoern Lemke               
//                                                         
// (C)opyright 2000-2013 Bjoern Lemke                        
//                                                         
// IMPLEMENTATION MODULE
//
// Class: Thread
//
// Description: all operations on POSIX threads
//
// Status: IMPLEMENTED
///////////////////////////////////////////////////////////////////////////////

#include <iostream>

#include "Thread.h"
#include "Exception.h"


#ifdef HAVE_MINGW32
#include <windows.h>
#else
#include <pthread.h>
static pthread_mutex_t mu = PTHREAD_MUTEX_INITIALIZER;
#endif

void* Thread::task(void* pThis)
{
   void* res = ((Thread*)pThis)->job( ((Thread*)pThis)->getArg() );
#ifndef HAVE_MINGW32
   pthread_exit(NULL);
#endif
   return res;
}

Thread::Thread()
{   
}

Thread::~Thread()
{
}

long Thread::start(void *arg)
{
    _arg = arg;


#ifdef HAVE_MINGW32
    unsigned long tid;
    HANDLE hThread = CreateThread(NULL, 2000, (LPTHREAD_START_ROUTINE)task, this, 9, &tid);
#else
    pthread_t tid;
   
    pthread_create(&tid, NULL, Thread::task,
		   (void *)this);
#endif
    _tid = (long)tid;

    return _tid;

}

void Thread::cancel()
{   
#ifdef HAVE_MINGW32
#else
    pthread_cancel((pthread_t)_tid);
#endif
}


void Thread::join(long tid)
{    
#ifdef HAVE_MINGW32
#else
    pthread_join((pthread_t)tid, NULL); 
#endif
}

void Thread::P()
{
#ifdef HAVE_MINGW32
#else
    if ( pthread_mutex_lock(&mu) != 0 )
	throw Exception(EXLOC, "pthread_mutex_lock system error");
#endif
}

void Thread::V()
{
#ifdef HAVE_MINGW32
#else
    if ( pthread_mutex_unlock(&mu) != 0 )
	throw Exception(EXLOC, "pthread_mutex_unlock system error");
#endif
}
